/*
 * Copyright (C) 2002 Roman Zippel <zippel@linux-m68k.org>
 * Released under the terms of the GNU GPL v2.0.
 */

#ifndef EXPR_H
#define EXPR_H

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

struct file {
	struct file *next;
	struct file *parent;
	struct statement *stmt;
	struct statement *last_stmt;
	char *name;
	int lineno;
	int flags;
};

#define FILE_BUSY		0x0001
#define FILE_SCANNED		0x0002
#define FILE_PRINTED		0x0004

struct symbol {
	struct symbol *next;
	char *name;
	char *help;
	int type;
	struct symbol *value, *def_value, *max_value;
	int flags;
	struct statement *stmt;
	struct expr *dep, *dep2;
	struct menu *menu;
};

#define for_all_symbols(i, sym) for (i = 0; i < 257; i++) for (sym = symbol_hash[i]; sym; sym = sym->next) if (sym->type != SYMBOL_OTHER)

#define SYMBOL_UNKNOWN		0
#define SYMBOL_BOOLEAN		1
#define SYMBOL_TRISTATE		2
#define SYMBOL_INT		3
#define SYMBOL_HEX		4
#define SYMBOL_STRING		5
#define SYMBOL_OTHER		6

#define SYMBOL_YES		0x0001
#define SYMBOL_MOD		0x0002
#define SYMBOL_NO		0x0004
#define SYMBOL_CONST		0x0007
#define SYMBOL_CHECK		0x0008
#define SYMBOL_CHOICE		0x0010
#define SYMBOL_PRINTED		0x0020
#define SYMBOL_DEP		0x0040
#define SYMBOL_OPTIONAL		0x0080
#define SYMBOL_WRITE		0x0100

struct statement {
	struct statement *next;
	struct symbol *sym;
	int token;
	char *prompt;
	struct symbol *def;
	struct expr *dep;
	struct expr *dep2;
	struct file *file;
	struct statement *next_pos;
	int lineno;
};

struct expr {
	int token;
	union {
		struct expr *expr;
		struct symbol *sym;
	} left, right;
};

#define T_WORD		300
#define T_UNEQUAL	301
#define T_OR		302
#define T_AND		303
#define T_IF		304
#define T_ELSE		305

struct menu {
	struct menu *next;
	struct menu *parent;
	struct menu *list;
	struct statement *stmt;
	struct statement *prompt;
	struct symbol *sym;
	void *data;
};

extern struct symbol *symbol_hash[257];
extern struct menu rootmenu;
extern struct file *file_list;
extern struct file *current_file;
struct file *lookup_file(char *name);

extern struct symbol symbol_yes, symbol_no, symbol_mod;
extern struct symbol *modules_sym;
extern int cdebug;
extern int print_type;
struct expr *alloc_expr_symbol(struct symbol *sym);
struct expr *alloc_expr_one(int token, struct expr *ce);
struct expr *alloc_expr_two(int token, struct expr *e1, struct expr *e2);
struct expr *alloc_expr_comp(int token, struct symbol *s1, struct symbol *s2);
struct expr *copy_expr(struct expr *org);
void free_expr(struct expr *e);
struct symbol *calc_value_and(struct symbol *max1, struct symbol *max2);
struct symbol *calc_value_or(struct symbol *max1, struct symbol *max2);
struct symbol *calc_value(struct expr *e);
struct symbol *calc_sym_value(struct symbol *sym);
void clear_sym_deps(void);
int contains_symbol(struct expr *dep, struct symbol *sym);

void fprint_expr(FILE *out, struct expr *e, int prevtoken);
void print_expr(int mask, struct expr *e, int prevtoken);

static inline int expr_is_yes(struct expr *e)
{
	return !e || (e->token == T_WORD && e->left.sym == &symbol_yes);
}

static inline int expr_is_no(struct expr *e)
{
	return e && (e->token == T_WORD && e->left.sym == &symbol_no);
}

#ifdef __cplusplus
}
#endif

#endif /* EXPR_H */
