/*
 * Copyright (C) 2002 Roman Zippel <zippel@linux-m68k.org>
 * Released under the terms of the GNU GPL v2.0.
 */

#include <qapplication.h>
#include <qvbox.h>
#include <qsplitter.h> 
#include <qlistview.h>
#include <qtextview.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include "zconf.h"
#include "qconf.h"

#include "qconf.moc"

static int showAll;

template <class P>
void updateMenuList(P *parent, struct menu *menu)
{
	struct menu *child;
	struct symbol *sym;
	ConfigItem *last;
	bool state;

	last = 0;
	for (child = menu->list; child; child = child->next) {
		sym = child->sym;
		if (sym) {
			calc_sym_value(sym);
			state = sym->max_value != &symbol_no;
		} else
			state = calc_value(child->stmt->dep) != &symbol_no;

		if (showAll || state) {
			if (!child->data)
				child->data = new ConfigItem(parent, last, child);
			last = (ConfigItem*)child->data;
			last->setEnabled(state);
			last->updateMenu();
		} else {
			if (child->data) {
				struct menu *m;

				delete (ConfigItem*)child->data;
				last = 0;
				for (m = menu->list; m != child; m = m->next)
					if (m->data)
						last = (ConfigItem*)m->data;
			}
		}
	}
}

void ConfigItem::okRename(int col)
{
	QListViewItem::okRename(col);
	free(menu->prompt->def->name);
	menu->prompt->def->name = strdup(text(4).latin1());
}

void ConfigItem::updateMenu(void)
{
	struct statement *stmt;
	struct symbol *sym;
	int type;

	sym = menu->sym;
	if ((stmt = menu->stmt) && !sym)
		goto update_list;

	type = sym->type;
	if ((sym->flags & SYMBOL_CHOICE) && !stmt)
		type = SYMBOL_BOOLEAN;
	if (type == SYMBOL_TRISTATE && calc_sym_value(modules_sym) == &symbol_no)
		type = SYMBOL_BOOLEAN;
	switch (type) {
	case SYMBOL_BOOLEAN:
		if (sym->flags & SYMBOL_OPTIONAL) {
			if (sym->value == &symbol_no) {
				setText(1, "N");
				setText(3, "_");
			} else {
				setText(1, "_");
				setText(3, "Y");
			}
		} else {
			setText(1, NULL);
			setText(3, NULL);
		}
		setText(2, NULL);
		break;
	case SYMBOL_TRISTATE:
		if (sym->flags & SYMBOL_OPTIONAL) {
			if (sym->value == &symbol_no)
				setText(1, "N");
			else
				setText(1, "_");
		} else
			setText(1, NULL);
		if (sym->max_value != &symbol_no) {
			if (sym->value == &symbol_mod)
				setText(2, "M");
			else
				setText(2, "_");
		} else
			setText(2, NULL);
		if (sym->max_value == &symbol_yes) {
			if (sym->value == &symbol_yes)
				setText(3, "Y");
			else
				setText(3, "_");
		} else
			setText(3, NULL);
		break;
	case SYMBOL_INT:
	case SYMBOL_HEX:
	case SYMBOL_STRING:
		if (sym->value)
			setText(4, sym->value->name);
		setRenameEnabled(4, TRUE);
		break;
	}

update_list:
	updateMenuList(this, menu);
}

void ConfigItem::init(void)
{
	struct symbol *sym;
	struct statement *stmt;

	menu->data = this;
	if ((stmt = menu->stmt)) {
		if (stmt->prompt)
			setText(0, stmt->prompt);
	} else if ((sym = menu->sym)) {
		if (!menu->prompt) {
			setText(0, sym->name);
			setEnabled(FALSE);
		} else
			setText(0, menu->prompt->prompt);
	}
	updateMenu();
}

ConfigList::ConfigList(QWidget *p)
	: QListView(p)
{
	setSorting(-1);
	setRootIsDecorated(TRUE);
	addColumn("prompt");
	addColumn("n");
	addColumn("m");
	addColumn("y");
	addColumn("data");

	connect(this, SIGNAL(clicked(QListViewItem*, const QPoint&, int)),
		SLOT(setValue(QListViewItem*, const QPoint&, int)));
	connect(this, SIGNAL(spacePressed(QListViewItem*)),
		SLOT(changeValue(QListViewItem*)));

	updateMenu();
}

void ConfigList::updateMenu(void)
{
	clear_sym_deps();
	updateMenuList(this, &rootmenu);
}

void ConfigList::setValue(QListViewItem* i, const QPoint& pnt, int c)
{
	ConfigItem *item;
	struct symbol *sym;
	int type;

	if (!i)
		return;

	item = (ConfigItem *)i;
	sym = item->menu->sym;
	if (!sym)
		return;

	type = sym->type;
	if (type == SYMBOL_TRISTATE && calc_sym_value(modules_sym) == &symbol_no)
		type = SYMBOL_BOOLEAN;
	if ((sym->flags & SYMBOL_CHOICE) && !item->menu->stmt) {
		struct statement *stmt;
		for (stmt = sym->stmt; stmt->next; stmt = stmt->next)
			;
		if (c == 1 && stmt->sym->value == &symbol_mod) {
			sym->def_value = &symbol_no;
		}
		if (c == 3) {
			stmt->def = sym;
			sym->def_value = stmt->sym->value;
			if (sym->value == &symbol_no)
				item->setOpen(TRUE);
		}
	} else switch (type) {
	case SYMBOL_BOOLEAN:
		if (sym->flags & SYMBOL_OPTIONAL) {
			if (c == 1)
				sym->def_value = &symbol_no;
			if (c == 3) {
				sym->def_value = &symbol_yes;
				if (sym->value == &symbol_no)
					item->setOpen(TRUE);
			}
		}
		break;
	case SYMBOL_TRISTATE:
		if (c == 1 && (sym->flags & SYMBOL_OPTIONAL))
			sym->def_value = &symbol_no;
		if (c == 2) {
			sym->def_value = &symbol_mod;
			if (sym->value == &symbol_no)
				item->setOpen(TRUE);
		}
		if (c == 3 && sym->max_value == &symbol_yes) {
			sym->def_value = &symbol_yes;
			if (sym->value == &symbol_no)
				item->setOpen(TRUE);
		}
		break;
	case SYMBOL_INT:
	case SYMBOL_HEX:
	case SYMBOL_STRING:
		if (c == 4)
			item->startRename(4);
		break;
	}

	updateMenu();
}

void ConfigList::changeValue(QListViewItem* i)
{
	ConfigItem *item;
	struct symbol *sym, *val;
	struct menu *menu;
	int type;

	if (!i)
		return;

	item = (ConfigItem *)i;
	menu = item->menu;
	sym = menu->sym;
	if (!sym) {
		item->setOpen(!item->isOpen());
		return;
	}

	val = sym->value;
	type = sym->type;
	if (type == SYMBOL_TRISTATE && calc_sym_value(modules_sym) == &symbol_no)
		type = SYMBOL_BOOLEAN;
	if ((sym->flags & SYMBOL_CHOICE) && !menu->stmt) {
		struct statement *stmt;
		for (stmt = sym->stmt; stmt->next; stmt = stmt->next)
			;
		if (val == &symbol_no) {
			stmt->def = sym;
			sym->def_value = stmt->sym->value;
			item->setOpen(TRUE);
		} else if (stmt->sym->value == &symbol_mod)
			sym->def_value = &symbol_no;
	} else switch (type) {
	case SYMBOL_BOOLEAN:
		if (sym->flags & SYMBOL_OPTIONAL) {
			if (val == &symbol_no) {
				sym->def_value = &symbol_yes;
				item->setOpen(TRUE);
			} else
				sym->def_value = &symbol_no;
		} else
			item->setOpen(!item->isOpen());
		break;
	case SYMBOL_TRISTATE:
		if (val == &symbol_no) {
			sym->def_value = &symbol_mod;
			item->setOpen(TRUE);
		} else if (val == &symbol_mod) {
			if (sym->max_value == &symbol_yes)
				sym->def_value = &symbol_yes;
			else
				sym->def_value = &symbol_no;
		} else if (val == &symbol_yes) {
			if (sym->flags & SYMBOL_OPTIONAL)
				sym->def_value = &symbol_no;
			else
				sym->def_value = &symbol_mod;
		}
		break;
	case SYMBOL_INT:
	case SYMBOL_HEX:
	case SYMBOL_STRING:
		item->startRename(4);
		break;
	}

	updateMenu();
}

ConfigView::ConfigView(void)
{
	QMenuBar *menu;
	QSplitter *splitter;
	QPopupMenu *config;
	ConfigList *configList;

	menu = new QMenuBar(this);
	splitter = new QSplitter(this);
	splitter->setOrientation(QSplitter::Vertical);

	config = new QPopupMenu(this);
	config->setFocusPolicy(QWidget::ClickFocus);
	config->insertItem("&quit", this, SLOT(close()), CTRL+Key_Q );
	menu->insertItem("&config", config);

	configList = new ConfigList(splitter);
	helpText = new QTextView(splitter);

	setTabOrder(configList, helpText);
	configList->setFocus();

	connect(configList, SIGNAL(selectionChanged (QListViewItem*)),
		SLOT(setHelp(QListViewItem*)));
}

void ConfigView::setHelp(QListViewItem* i)
{
	ConfigItem *item;
	struct symbol *sym;
	struct menu *menu;

	if (!i)
		return;

	item = (ConfigItem *)i;
	menu = item->menu;

	sym = menu->sym;
	if (sym)
		helpText->setText(sym->help);
	else
		helpText->setText(NULL);
}

void ConfigView::closeEvent(QCloseEvent *e)
{
	QMessageBox mb("qconf", "Save configuration?", QMessageBox::NoIcon,
			QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel | QMessageBox::Escape);
	switch (mb.exec()) {
	case QMessageBox::Yes:
		writeconf();
	case QMessageBox::No:
		e->accept();
		break;
	case QMessageBox::Cancel:
		e->ignore();
		break;
	}
}

int main(int ac, char **av)
{
	QApplication a(ac, av);
	ConfigView *v;

	if (ac > 1 && av[1][0] == '-') {
		switch (av[1][1]) {
		case 'a':
			showAll = 1;
			break;
		case 'h':
		case '?':
			printf("%s -a\n", av[0]);
			exit(0);
		}
	}
	zconf_initscan("config.new");
	zconfparse();
	modules_sym = lookup_symbol("MODULES");
	readconf();
	v = new ConfigView();

	//zconfdump(stdout);
	v->show();
	a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));
	a.exec();
	return 0;
}
